@@ -146,12 +146,13 @@ def revolve(linestring,
     faces = (stacked + offset) % len(vertices)
 
     # create the mesh from our vertices and faces
-    mesh = Trimesh(vertices=vertices, faces=faces)
+    mesh = Trimesh(vertices=vertices, faces=faces,
+                   **kwargs)
 
     # strict checks run only in unit tests
     if (tol.strict and
-        np.allclose(radius[[0, -1]], 0.0) or
-            np.allclose(linestring[0], linestring[-1])):
+            (np.allclose(radius[[0, -1]], 0.0) or
+             np.allclose(linestring[0], linestring[-1]))):
         # if revolved curve starts and ends with zero radius
         # it should really be a valid volume, unless the sign
         # reversed on the input linestring
@@ -227,7 +228,8 @@ def sweep_polygon(polygon,
 
     # Extract 2D vertices and triangulation
     verts_2d = np.array(polygon.exterior)[:-1]
-    base_verts_2d, faces_2d = triangulate_polygon(polygon, **kwargs)
+    base_verts_2d, faces_2d = triangulate_polygon(
+        polygon, **kwargs)
     n = len(verts_2d)
 
     # Create basis for first planar polygon cap
@@ -298,7 +300,7 @@ def sweep_polygon(polygon,
     vecs = verts_3d - path[-1]
     coords = np.c_[np.einsum('ij,j->i', vecs, x),
                    np.einsum('ij,j->i', vecs, y)]
-    base_verts_2d, faces_2d = triangulate_polygon(Polygon(coords))
+    base_verts_2d, faces_2d = triangulate_polygon(Polygon(coords), **kwargs)
     base_verts_3d = (np.einsum('i,j->ij', base_verts_2d[:, 0], x) +
                      np.einsum('i,j->ij', base_verts_2d[:, 1], y)) + path[-1]
     faces = np.vstack((faces, faces_2d + len(vertices)))
@@ -581,9 +583,8 @@ def box(extents=None, transform=None, **kwargs):
       Mesh of a cuboid
     """
     # vertices of the cube
-    vertices = [0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
-                1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1]
-    vertices = np.array(vertices,
+    vertices = np.array([0, 0, 0, 0, 0, 1, 0, 1, 0, 0, 1,
+                         1, 1, 0, 0, 1, 0, 1, 1, 1, 0, 1, 1, 1],
                         order='C',
                         dtype=np.float64).reshape((-1, 3))
     vertices -= 0.5
@@ -594,6 +595,8 @@ def box(extents=None, transform=None, **kwargs):
         if extents.shape != (3,):
             raise ValueError('Extents must be (3,)!')
         vertices *= extents
+    else:
+        extents = np.asarray((1.0, 1.0, 1.0), dtype=np.float64)
 
     # hardcoded face indices
     faces = [1, 3, 0, 4, 1, 0, 0, 3, 2, 2, 4, 0, 1, 7, 3, 5, 1, 4,
@@ -606,6 +609,12 @@ def box(extents=None, transform=None, **kwargs):
                                  order='C',
                                  dtype=np.float64).reshape(-1, 3)
 
+    if 'metadata' not in kwargs:
+        kwargs['metadata'] = dict()
+    kwargs['metadata'].update(
+        {'shape': 'box',
+         'extents': extents})
+
     box = Trimesh(vertices=vertices,
                   faces=faces,
                   face_normals=face_normals,
@@ -678,6 +687,8 @@ def icosphere(subdivisions=3, radius=1.0, color=None):
     ico._validate = True
     if color is not None:
         ico.visual.face_colors = color
+    ico.metadata.update({'shape': 'sphere',
+                         'radius': radius})
     return ico
 
 
@@ -707,7 +718,7 @@ def uv_sphere(radius=1.0,
        Mesh of UV sphere with specified parameters
     """
 
-    count = np.array(count, dtype=np.int)
+    count = np.array(count, dtype=np.int64)
     count += np.mod(count, 2)
     count[1] *= 2
 
@@ -751,7 +762,9 @@ def uv_sphere(radius=1.0,
 
     # we save a lot of time by not processing again
     # since we did some bookkeeping mesh is watertight
-    mesh = Trimesh(vertices=vertices, faces=faces, process=False)
+    mesh = Trimesh(vertices=vertices, faces=faces, process=False,
+                   metadata={'shape': 'sphere',
+                             'radius': radius})
     return mesh
 
 
@@ -780,7 +793,7 @@ def capsule(height=1.0,
     """
     height = float(height)
     radius = float(radius)
-    count = np.array(count, dtype=np.int)
+    count = np.array(count, dtype=np.int64)
     count += np.mod(count, 2)
 
     # create a theta where there is a double band around the equator
@@ -800,6 +813,9 @@ def capsule(height=1.0,
 
     top = capsule.vertices[:, 2] > tol.zero
     capsule.vertices[top] += [0, 0, height]
+    capsule.metadata.update({'shape': 'capsule',
+                             'height': height,
+                             'radius': radius})
 
     return capsule
 
@@ -835,6 +851,12 @@ def cone(radius,
                   [radius, 0],
                   [0, height]]
     # revolve the profile to create a cone
+    if 'metadata' not in kwargs:
+        kwargs['metadata'] = dict()
+    kwargs['metadata'].update(
+        {'shape': 'cone',
+         'radius': radius,
+         'height': height})
     cone = revolve(linestring=linestring,
                    sections=sections,
                    transform=transform,
@@ -886,10 +908,17 @@ def cylinder(radius,
                   [radius, -half],
                   [radius, half],
                   [0, half]]
+    if 'metadata' not in kwargs:
+        kwargs['metadata'] = dict()
+    kwargs['metadata'].update(
+        {'shape': 'cylinder',
+         'height': height,
+         'radius': radius})
     # generate cylinder through simple revolution
     return revolve(linestring=linestring,
                    sections=sections,
-                   transform=transform)
+                   transform=transform,
+                   **kwargs)
 
 
 def annulus(r_min,
@@ -948,6 +977,14 @@ def annulus(r_min,
                   [r_min, half],
                   [r_min, -half]]
 
+    if 'metadata' not in kwargs:
+        kwargs['metadata'] = dict()
+    kwargs['metadata'].update(
+        {'shape': 'annulus',
+         'r_min': r_min,
+         'r_max': r_max,
+         'height': height})
+
     # revolve the curve
     annulus = revolve(linestring=linestring,
                       sections=sections,
@@ -1124,8 +1161,12 @@ def camera_marker(camera,
       Contains Trimesh and Path3D objects which can be visualized
     """
 
+    # create sane origin size from marker height
+    if origin_size is None:
+        origin_size = marker_height / 10.0
+
     # append the visualizations to an array
-    meshes = [axis(origin_size=marker_height / 10.0)]
+    meshes = [axis(origin_size=origin_size)]
 
     try:
         # path is a soft dependency
@@ -1136,10 +1177,6 @@ def camera_marker(camera,
                     exc_info=True)
         return meshes
 
-    # create sane origin size from marker height
-    if origin_size is None:
-        origin_size = marker_height / 10.0
-
     # calculate vertices from camera FOV angles
     x = marker_height * np.tan(np.deg2rad(camera.fov[0]) / 2.0)
     y = marker_height * np.tan(np.deg2rad(camera.fov[1]) / 2.0)
