@@ -4,18 +4,23 @@ import numpy as np
 
 from ..constants import log
 from .. import util
+from .. import resolvers
 
 from .urdf import export_urdf  # NOQA
 from .gltf import export_glb, export_gltf
 from .obj import export_obj
 from .off import _off_exporters
 from .stl import export_stl, export_stl_ascii
-from .ply import _ply_exporters
+from .ply import export_ply, _ply_exporters
 from .dae import _collada_exporters
 from .xyz import _xyz_exporters
 
 
-def export_mesh(mesh, file_obj, file_type=None, **kwargs):
+def export_mesh(mesh,
+                file_obj,
+                file_type=None,
+                resolver=None,
+                **kwargs):
     """
     Export a Trimesh object to a file- like object, or to a filename
 
@@ -25,6 +30,8 @@ def export_mesh(mesh, file_obj, file_type=None, **kwargs):
       Where should mesh be exported to
     file_type : str or None
       Represents file type (eg: 'stl')
+    resolver : None or trimesh.resolvers.Resolver
+      Resolver to write referenced assets to
 
     Returns
     ----------
@@ -48,6 +55,9 @@ def export_mesh(mesh, file_obj, file_type=None, **kwargs):
             # get full path of file before opening
             file_path = os.path.abspath(os.path.expanduser(file_obj))
             file_obj = open(file_path, 'wb')
+            if resolver is None:
+                # create a resolver which can write files to the path
+                resolver = resolvers.FilePathResolver(file_path)
 
     # make sure file type is lower case
     file_type = str(file_type).lower()
@@ -65,13 +75,16 @@ def export_mesh(mesh, file_obj, file_type=None, **kwargs):
         # if the mesh has faces log the number
         log.debug('Exporting %d faces as %s', len(mesh.faces),
                   file_type.upper())
+
+    # OBJ files save assets everywhere
+    if file_type == 'obj':
+        kwargs['resolver'] = resolver
     export = _mesh_exporters[file_type](mesh, **kwargs)
 
     if hasattr(file_obj, 'write'):
         result = util.write_encoded(file_obj, export)
     else:
         result = export
-
     if was_opened:
         file_obj.close()
 
@@ -92,10 +105,10 @@ def export_dict(mesh, encoding=None):
 
     Parameters
     ------------
-    mesh : Trimesh object
-             Mesh to be exported
-    encoding : str, or None
-                 'base64'
+    mesh : trimesh.Trimesh
+      Mesh to be exported
+    encoding : str or None
+      Such as 'base64'
 
     Returns
     -------------
@@ -176,6 +189,94 @@ def scene_to_dict(scene, use_base64=False):
     return export
 
 
+def export_scene(scene,
+                 file_obj,
+                 file_type=None,
+                 resolver=None,
+                 **kwargs):
+    """
+    Export a snapshot of the current scene.
+
+    Parameters
+    ----------
+    file_obj : str, file-like, or None
+      File object to export to
+    file_type : str or None
+      What encoding to use for meshes
+      IE: dict, dict64, stl
+
+    Returns
+    ----------
+    export : bytes
+      Only returned if file_obj is None
+    """
+    # if we weren't passed a file type extract from file_obj
+    if file_type is None:
+        if util.is_string(file_obj):
+            file_type = str(file_obj).split('.')[-1]
+        else:
+            raise ValueError('file_type not specified!')
+
+    # always remove whitepace and leading characters
+    file_type = file_type.strip().lower().lstrip('.')
+
+    # now handle our different scene export types
+    if file_type == 'gltf':
+        data = export_gltf(scene, **kwargs)
+    elif file_type == 'glb':
+        data = export_glb(scene, **kwargs)
+    elif file_type == 'dict':
+        data = scene_to_dict(scene)
+    elif file_type == 'obj':
+        if resolver is None and util.is_string(file_obj):
+            resolver = resolvers.FilePathResolver(file_obj)
+        data = export_obj(scene, resolver=resolver)
+    elif file_type == 'dict64':
+        data = scene_to_dict(scene, use_base64=True)
+    elif file_type == 'svg':
+        from trimesh.path.exchange import svg_io
+        data = svg_io.export_svg(scene, **kwargs)
+    elif file_type == 'ply':
+        data = export_ply(scene)
+    elif file_type == 'stl':
+        data = export_stl(scene)
+    else:
+        raise ValueError(
+            'unsupported export format: {}'.format(file_type))
+
+    # now write the data or return bytes of result
+    if isinstance(data, dict):
+        # GLTF files return a dict-of-bytes as they
+        # represent multiple files so create a filepath
+        # resolver and write the files if someone passed
+        # a path we can write to.
+        if resolver is None and util.is_string(file_obj):
+            resolver = resolvers.FilePathResolver(file_obj)
+            # the requested "gltf"
+            bare_path = os.path.split(file_obj)[-1]
+            for name, blob in data.items():
+                if name == 'model.gltf':
+                    # write the root data to specified file
+                    resolver.write(bare_path, blob)
+                else:
+                    # write the supporting files
+                    resolver.write(name, blob)
+        return data
+
+    if hasattr(file_obj, 'write'):
+        # if it's just a regular file object
+        return util.write_encoded(file_obj, data)
+    elif util.is_string(file_obj):
+        # assume strings are file paths
+        file_path = os.path.expanduser(
+            os.path.abspath(file_obj))
+        with open(file_path, 'wb') as f:
+            util.write_encoded(f, data)
+
+    # no writeable file object so return data
+    return data
+
+
 def export_json(mesh):
     blob = export_dict(mesh, encoding='base64')
     export = json.dumps(blob)
