@@ -164,10 +164,12 @@ def load_obj(file_obj,
             log.debug('faces have mixed data, using slow fallback!')
             faces, faces_tex, faces_norm = _parse_faces_fallback(face_lines)
 
-        # TODO: name usually falls back to something useless
-        name = current_object
-        if name is None or len(name) == 0 or name in geometry:
-            name = '{}_{}'.format(name, util.unique_id())
+        if group_material:
+            name = material
+        else:
+            name = current_object
+            if name is None or len(name) == 0 or name in geometry:
+                name = '{}_{}'.format(name, util.unique_id())
 
         # try to get usable texture
         mesh = kwargs.copy()
@@ -750,7 +752,8 @@ def export_obj(mesh,
                return_texture=False,
                write_texture=True,
                resolver=None,
-               digits=8):
+               digits=8,
+               header='https://github.com/mikedh/trimesh'):
     """
     Export a mesh as a Wavefront OBJ file.
     TODO: scenes with textured meshes
@@ -774,13 +777,14 @@ def export_obj(mesh,
       Resolver which can write referenced text objects
     digits : int
       Number of digits to include for floating point
+    header : str or None
+      Header string for top of file or None for no header.
 
     Returns
     -----------
     export : str
       OBJ format output
     texture : dict
-      [OPTIONAL]
       Contains files that need to be saved in the same
       directory as the exported mesh: {file name : bytes}
     """
@@ -847,7 +851,7 @@ def export_obj(mesh,
                 log.debug('failed to convert vertex normals',
                           exc_info=True)
 
-        if include_texture and hasattr(mesh.visual, 'uv'):
+        if include_texture and len(np.shape(getattr(mesh.visual, 'uv', None))) == 2:
             try:
                 material = mesh.visual.material
                 if hasattr(material, 'to_simple'):
@@ -891,8 +895,9 @@ def export_obj(mesh,
         # add this object
         objects.append('\n'.join(export))
 
-    # add a created-with header to the top of the file
-    objects.appendleft('# https://github.com/mikedh/trimesh')
+    if header is not None:
+        # add a created-with header to the top of the file
+        objects.appendleft('# {}'.format(header))
     # combine elements into a single string
     text = '\n'.join(objects)
 
