@@ -1,7 +1,7 @@
-import numpy as np
-
 import copy
 
+import numpy as np
+
 from .base import Visuals
 from . import color
 
@@ -16,7 +16,8 @@ class TextureVisuals(Visuals):
     def __init__(self,
                  uv=None,
                  material=None,
-                 image=None):
+                 image=None,
+                 face_materials=None):
         """
         Store a single material and per-vertex UV coordinates
         for a mesh.
@@ -35,9 +36,9 @@ class TextureVisuals(Visuals):
         """
 
         # store values we care about enough to hash
-        self._data = caching.DataStore()
+        self.vertex_attributes = caching.DataStore()
         # cache calculated values
-        self._cache = caching.Cache(self._data.fast_hash)
+        self._cache = caching.Cache(self.vertex_attributes.fast_hash)
 
         # should be (n, 2) float
         self.uv = uv
@@ -52,9 +53,11 @@ class TextureVisuals(Visuals):
             # if passed assign
             self.material = material
 
+        self.face_materials = face_materials
+
     def _verify_crc(self):
         """
-        Dump the cache if anything in self._data has changed.
+        Dump the cache if anything in self.vertex_attributes has changed.
         """
         self._cache.verify()
 
@@ -90,9 +93,9 @@ class TextureVisuals(Visuals):
         Returns
         --------------
         crc : int
-          Hash of items in self._data
+          Hash of items in self.vertex_attributes
         """
-        return self._data.crc()
+        return self.vertex_attributes.crc()
 
     @property
     def uv(self):
@@ -104,9 +107,7 @@ class TextureVisuals(Visuals):
         uv : (n, 2) float
           Pixel position per- vertex
         """
-        if 'uv' in self._data:
-            return self._data['uv']
-        return None
+        return self.vertex_attributes.get('uv', None)
 
     @uv.setter
     def uv(self, values):
@@ -119,12 +120,12 @@ class TextureVisuals(Visuals):
           Pixel locations on a texture per- vertex
         """
         if values is None:
-            self._data.clear()
+            self.vertex_attributes['uv'] = None
         else:
-            self._data['uv'] = np.asanyarray(
+            self.vertex_attributes['uv'] = np.asanyarray(
                 values, dtype=np.float64)
 
-    def copy(self):
+    def copy(self, uv=None):
         """
         Return a copy of the current TextureVisuals object.
 
@@ -133,12 +134,14 @@ class TextureVisuals(Visuals):
         copied : TextureVisuals
           Contains the same information in a new object
         """
-        uv = self.uv
+        if uv is None:
+            uv = self.uv
         if uv is not None:
             uv = uv.copy()
         copied = TextureVisuals(
             uv=uv,
-            material=copy.deepcopy(self.material))
+            material=self.material.copy(),
+            face_materials=copy.copy(self.face_materials))
 
         return copied
 
@@ -162,18 +165,42 @@ class TextureVisuals(Visuals):
         """
         Get a copy of
         """
-        return self.copy()
+        if self.uv is not None:
+            indices = np.unique(self.mesh.faces[face_index].flatten())
+            return self.copy(self.uv[indices])
+        else:
+            return self.copy()
 
     def update_vertices(self, mask):
         """
         Apply a mask to remove or duplicate vertex properties.
+
+        Parameters
+        ------------
+        mask : (len(vertices),) bool or (n,) int
+          Mask which can be used like: `vertex_attribute[mask]`
         """
-        if self.uv is not None:
-            self.uv = self.uv[mask]
+        # collect updated masked values
+        updates = {}
+        for key, value in self.vertex_attributes.items():
+            # DataStore will convert None to zero-length array
+            if len(value) == 0:
+                continue
+            try:
+                # store the update
+                updates[key] = value[mask]
+            except BaseException:
+                # usual reason is an incorrect size or index
+                util.log.warning('failed to update visual: `{}`'.format(key))
+        # clear all values from the vertex attributes
+        self.vertex_attributes.clear()
+        # apply the updated values
+        self.vertex_attributes.update(updates)
 
     def update_faces(self, mask):
         """
-        Apply a mask to remove or duplicate face properties
+        Apply a mask to remove or duplicate face properties,
+        not applicable to texture visuals.
         """
         pass
 
@@ -197,7 +224,7 @@ class TextureVisuals(Visuals):
         return concatenate(self, others)
 
 
-def unmerge_faces(faces, *args):
+def unmerge_faces(faces, *args, **kwargs):
     """
     Textured meshes can come with faces referencing vertex
     indices (`v`) and an array the same shape which references
@@ -214,6 +241,8 @@ def unmerge_faces(faces, *args):
     *args : (n, d) int
       Various references of corresponding values
       This is usually UV coordinates or normal indexes
+    maintain_faces : bool
+      Do not alter original faces and return no-op masks.
 
     Returns
     -------------
@@ -225,12 +254,42 @@ def unmerge_faces(faces, *args):
       A mask to apply to vt array to get matching UV coordinates
       Returns as many of these as args were passed
     """
+    # unfortunately Python2 doesn't let us put named kwargs
+    # after an `*args` sequence so we have to do this ugly get
+    maintain_faces = kwargs.get('maintain_faces', False)
+
+    # don't alter faces
+    if maintain_faces:
+        # start with not altering faces at all
+        result = [faces]
+        # find the maximum index referenced by faces
+        max_idx = faces.max()
+        # add a vertex mask which is just ordered
+        result.append(np.arange(max_idx + 1))
+
+        # now given the order is fixed do our best on the rest of the order
+        for arg in args:
+            # create a mask of the attribute-vertex mapping
+            # note that these might conflict since we're not unmerging
+            masks = np.zeros((3, max_idx + 1), dtype=np.int64)
+            # set the mask using the unmodified face indexes
+            for i, f, a in zip(range(3), faces.T, arg.T):
+                masks[i][f] = a
+            # find the most commonly occurring attribute (i.e. UV coordinate)
+            # and use that index note that this is doing a float conversion
+            # and then median before converting back to int: could also do this as
+            # a column diff and sort but this seemed easier and is fast enough
+            result.append(np.median(masks, axis=0).astype(np.int64))
+
+        return result
+
     # stack into pairs of (vertex index, texture index)
     stackable = [np.asanyarray(faces).reshape(-1)]
     # append multiple args to the correlated stack
     # this is usually UV coordinates (vt) and normals (vn)
     for arg in args:
         stackable.append(np.asanyarray(arg).reshape(-1))
+
     # unify them into rows of a numpy array
     stack = np.column_stack(stackable)
     # find unique pairs: we're trying to avoid merging
